/*
 *      @(#)VrmlLoader.java 1.14 99/02/11 18:02:52
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 *
 */


package com.sun.j3d.loaders.vrml97;

import com.sun.j3d.loaders.IncorrectFormatException;
import com.sun.j3d.loaders.ParsingErrorException;
import java.io.Reader;
import java.io.FileNotFoundException;
import java.lang.String;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.Reader;

/**
 * This is an implementation of the Java3D utility loader interface.
 * 
 * A VrmlScene can be loaded via URL or filesystem pathname (loading via a 
 * Reader object may be supported in a coming release).
 *
 * The "base" URL or pathname is the pathname used for relative paths in the 
 * WRL.   If the base is not specified it will be derived from the main URL or
 * pathname.
 *
 * @see Scene
 * @see com.sun.j3d.loaders.Loader
 * @see com.sun.j3d.loaders.Scene
 *
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 *
 */
public class VrmlLoader extends com.sun.j3d.loaders.LoaderBase {

    com.sun.j3d.loaders.vrml97.impl.Loader loader;

    public VrmlLoader(int flags) {
	super(flags);
	loader = new com.sun.j3d.loaders.vrml97.impl.Loader(null,
	    com.sun.j3d.loaders.vrml97.impl.Loader.LOAD_STATIC);
    }

    public VrmlLoader() {
	this(0);
    }

    public com.sun.j3d.loaders.Scene load(Reader reader) 
	    throws FileNotFoundException, IncorrectFormatException,
	    ParsingErrorException {
	com.sun.j3d.loaders.vrml97.impl.Scene scene;
	loader.setWorldURL(baseUrl, null);
	try {
	    scene = loader.load(reader);
	} catch (Exception e) {
	    // TODO: make a better handler
	    System.err.println("Exception loading URL: " + e);
	    e.printStackTrace(System.err);
	    throw new ParsingErrorException();
	}
	return new VrmlScene(scene);
    }

    public com.sun.j3d.loaders.Scene load(String pathname)
	    throws FileNotFoundException, IncorrectFormatException,
	    ParsingErrorException {
	
	URL url = pathToURL(pathname);
	loader.setWorldURL(pathToURL(basePath), url);
	return doLoad(url);
    }

    public com.sun.j3d.loaders.Scene load(URL url) 
	    throws FileNotFoundException, IncorrectFormatException,
	    ParsingErrorException {
	loader.setWorldURL(baseUrl, url);
	return doLoad(url);
    }

    private com.sun.j3d.loaders.Scene doLoad(URL url) {
	com.sun.j3d.loaders.vrml97.impl.Scene scene;
	try {
	    scene = loader.load(url);
	} catch (Exception e) {
	    // TODO: make a better handler
	    System.err.println("Exception loading URL: " + e);
	    e.printStackTrace(System.err);
	    throw new ParsingErrorException();
	}
	return new VrmlScene(scene);
    }

    // this method should only be used by methods that expect to only operate
    // on filesystem resident files, and those methods should not be used.
    private URL pathToURL(String path) {
	URL retval = null;
	if (path == null) {
	    return null;
	}
	
	// hack: for win32 we check drive specifier
	//       for solaris we check startsWith /

	// this really should all take a clean look at URL(context,spec) 

	if (!path.startsWith(java.io.File.separator) && (path.indexOf(':') != 1)){
	    path = System.getProperties().getProperty("user.dir") + '/' + path;
	}

	// switch from file separator to URL separator
	path = path.replace(java.io.File.separatorChar, '/');
	try {
	    retval = new URL("file:" + path);
	} catch (MalformedURLException me) {
	    System.out.println("Can't make a URL from path: " + path);
	}
	return retval;
    }

}
