/*
 *      @(#)BaseNode.java 1.21 98/11/05 20:34:02
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.sun.j3d.loaders.vrml97.impl;

public abstract class BaseNode 
{
    String defName;
    Browser browser;
    Loader loader;
    boolean implReady;
    javax.media.j3d.Node implNode;
    BaseNode parent;

    BaseNode(Loader loader) { 
	this.loader = loader;
	this.browser = loader.browser; 
	//System.out.println("Node " + this + " created");
    }

    public abstract String getType();
    public abstract Object clone();
    public abstract void notifyMethod(String s, double time);
    public abstract vrml.BaseNode wrap();
    public abstract Field getField(String fieldName);

    public Browser getBrowser() { return browser; }
    void initImpl() { 
	if (loader.debug)  {
	    System.out.println("BaseNode.initImpl() called on node " + 
		toStringId());
	}
	implNode = null; 
	implReady = true; 
    }
    // called by parent after node is added to parent
    void updateParent(javax.media.j3d.Node parentImpl) {};
    public javax.media.j3d.Node getImplNode() { return implNode; }
    void define(String defName) { 
	this.defName = defName; 
	//System.out.println("Node " + this + " def'd to name " + defName);
    }
    void registerUse(Scene scene) {}

    // subclasses which have triangles should define
    public int getNumTris() { return 0; }

    // subclasses should implement toStringBody() not toString()
    public final String toString() {
	String retval = "";
	if (defName != null) {
	    retval += "DEF " + defName + " ";
	}
	retval += toStringBody();
	return retval;
    }

    // use instead of toString() to get object id instead of VRML source
    public String toStringId() {
	return this.getClass().getName() + "@" + 
		Integer.toHexString(this.hashCode());
    }

    // this should be overridden by a method which outputs the VRML for the
    // node
    String toStringBody() {
	return toStringId();
    }
}
