/*
 *      @(#)Billboard.java 1.20 99/02/16 15:27:40
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.sun.j3d.loaders.vrml97.impl;

public class Billboard extends Transform {

    // exposedField 
    SFVec3f axisOfRotation;

    BillboardAxis implBillboard;

    public Billboard(Loader loader) {
	super(loader);
	
	axisOfRotation = new SFVec3f(0.0f,1.0f,0.0f);
	initBillboardFields();
	implBillboard = new BillboardAxis(this);
	notifyMethod("axisOfRotation",0.0);
    }

    Billboard(Loader loader, MFNode children, SFVec3f bboxCenter,
	SFVec3f  bboxSize,  SFVec3f aor ) {

	super(loader, children, bboxCenter, bboxSize, 
	    new SFVec3f(0.0f,0.0f,0.0f), 
	    new SFRotation(), new SFVec3f(1.0f,1.0f,1.0f), new SFRotation(), 
	    new SFVec3f(0.0f,0.0f,0.f) );
	axisOfRotation = aor;
	initBillboardFields();
	implBillboard = new BillboardAxis(this);
	notifyMethod("axisOfRotation",0.0);
    }

    public void initImpl() { 
	super.initImpl();
	super.impl.setCapability(javax.media.j3d.TransformGroup.ALLOW_TRANSFORM_WRITE);
	super.impl.setCapability(javax.media.j3d.TransformGroup.ALLOW_TRANSFORM_READ);
	super.impl.addChild(implBillboard);
	implBillboard.setTarget(super.impl);
	implBillboard.setSchedulingBoundingLeaf(loader.infiniteBoundingLeaf);
    }
    
    public void notifyMethod(String eventInName, double time) {
	if ( eventInName.equals("axisOfRotation")) {
	    float[] axis = axisOfRotation.value;
	    implBillboard.setAlignmentAxis(axis[0],axis[1],axis[2]);
	}
    }

    
    public Object clone() {
	return new Billboard(loader, 
		(MFNode)children.clone(),
	        (SFVec3f)bboxCenter.clone(),
	        (SFVec3f)bboxSize.clone(),
		(SFVec3f)axisOfRotation.clone() );
    }



    public String getType() { return "Billboard"; }

    void initBillboardFields() {
	axisOfRotation.init(this, FieldSpec, Field.EXPOSED_FIELD,
	    "axisOfRotation");
    }

}
