/*
 *      @(#)ImageTexture.java 1.22 98/11/05 20:34:32
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/* 
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.sun.j3d.loaders.vrml97.impl;
import javax.media.j3d.Texture;
import javax.media.j3d.Texture2D;
import javax.media.j3d.ImageComponent;
import javax.media.j3d.ImageComponent2D;
import java.awt.image.*;
import java.awt.geom.AffineTransform;
import java.awt.*;

public class ImageTexture extends Node implements TextureSrc , ImageObserver {

    // exposedField
    MFString url;	
    SFBool   repeatS;
    SFBool   repeatT;
    
    Texture2D 	impl;
    ImageComponent2D imageComponent;
    Toolkit toolkit;
    BufferedImage image;
    Image mimage;
    int width;
    int height;

    public ImageTexture(Loader loader) {
	super(loader);
	url = new MFString();
	repeatS = new SFBool(true);
	repeatT = new SFBool(true);
	initFields();
    }

    ImageTexture(Loader loader, MFString url, SFBool repeatS, 
		SFBool repeatT) {
	super(loader);
	this.url = url;
	this.repeatS = repeatS;
	this.repeatT = repeatT;
	initFields();
    }

    void initImpl() {
	toolkit = Toolkit.getDefaultToolkit();
	impl = null;
	doChangeUrl();
	implReady = true;
    }

    public Object clone() {
	return new ImageTexture(loader, (MFString)url.clone(), 
		(SFBool)repeatS.clone(), (SFBool)repeatT.clone());
    }

    public void notifyMethod(String eventInName, double time) {
	if ( eventInName.equals("url") ) {
	    doChangeUrl();	
	} else if (eventInName.equals("repeatS")) {
	    if (impl != null) {
		if (repeatS.value == true) {
		    impl.setBoundaryModeS(Texture.WRAP);
		} else {
		    impl.setBoundaryModeS(Texture.CLAMP);
		}
	    }
	} else if (eventInName.equals("repeatT")) {
	    if (impl != null) {
		if (repeatT.value == true) {
		    impl.setBoundaryModeT(Texture.WRAP);
		} else {
		    impl.setBoundaryModeT(Texture.CLAMP);
		}
	    }
	}
    }
    
    void doChangeUrl() {
	if (url.strings == null) {
	    return; 
	} else if (url.strings.length > 0) { // this could be an absolute url though
	    PixelGrabber pg;
	    BufferedImage buf;
	    DirectColorModel colorModel;
	    WritableRaster raster;
	    int w,h,x,y;
	    int[] dat;
	    int[] tdat;
	    byte[] bytes = loader.getRelBytes(url.strings[0]);
	    mimage = toolkit.createImage(bytes);
	    w = mimage.getWidth(this);
	    h = mimage.getHeight(this);
	    Image aux = mimage.getScaledInstance(512,512,Image.SCALE_DEFAULT);
	    pg = new PixelGrabber(aux,0,0,512,512,true);
	    try {
		pg.grabPixels();
	    } catch (InterruptedException e) {;}
	    dat = (int[])pg.getPixels();
	    
	    colorModel = 
		new DirectColorModel(32,0xff000000,0xff0000,0xff00,0xff);
	    raster = colorModel.createCompatibleWritableRaster(512,512);

	    tdat = ((DataBufferInt)raster.getDataBuffer()).getData();

	    for(int i = 0; i < 512*512; i++) { 
	   	int p = dat[i];
		int a = (p >> 24) & 0xff;
		int r = (p >> 16) & 0xff;
		int g = (p >> 8) & 0xff;
		int b = (p) &0xff;
		tdat[i] = (r<<24)|(g<<16)|(b<<8)|a;
	    } 
	    buf = new BufferedImage(colorModel,raster,false,null);

	    imageComponent =  new ImageComponent2D(ImageComponent.FORMAT_RGBA,
		buf);
	    impl = new Texture2D(Texture.BASE_LEVEL, Texture.RGB, 
		imageComponent.getWidth(), imageComponent.getHeight());
	    impl.setMinFilter(Texture.BASE_LEVEL_LINEAR);
	    impl.setMagFilter(Texture.BASE_LEVEL_LINEAR);
	    impl.setImage(0, imageComponent);
	    impl.setEnable(true);
	}
    }

    public Texture getImplTexture() {
 	return impl;
    }

    public String getType() { return "ImageTexture"; }

    void initFields() {
	url.init(    this, FieldSpec, Field.EXPOSED_FIELD,"url");
	repeatS.init(this, FieldSpec, Field.EXPOSED_FIELD,"repeatS");
	repeatT.init(this, FieldSpec, Field.EXPOSED_FIELD,"repeatT");

    }	


    public boolean imageUpdate(Image img, int info, int x, int y, int w, int h ) {
	boolean moreUpdates=true;

	// new 
	Thread t = Thread.currentThread();
	t.setPriority(5);
	if ((( info & ImageObserver.WIDTH) != 0) &&
	    (( info & ImageObserver.HEIGHT) != 0)) {
		moreUpdates = false;
		this.width = w;
		this.height = h;
	}

	return moreUpdates;

    }


}	
