/*
 *      @(#)Proto.java 1.14 99/02/09 17:29:44
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.sun.j3d.loaders.vrml97.impl;
import java.util.Observer;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

public class Proto implements Namespace {

    String	name;
    Loader	loader;
    Vector	nodes = new Vector();
    public Hashtable FieldSpec = new Hashtable(4);
    Hashtable 	defTable = new Hashtable();
    Vector	routes = new Vector();
    ProtoInstance newInstance;

    Proto(Loader loader, String initName) {
	this.loader = loader;
	name = initName;
	if(loader.debug)System.out.println("PROTO "+initName);
    }

    String getName() {
	return name;
    }

    public void define(String defName, BaseNode node) {
        defTable.put(defName, node);
    }

    public BaseNode use(String defName) {
	BaseNode orgNode = (BaseNode) defTable.get(defName);
        return new ProtoUseNode(loader, this, orgNode);
    }

    public void addRoute(BaseNode fromNode, String fromEventOut,
            BaseNode toNode, String toEventIn) {
	if (fromNode instanceof ProtoUseNode) {
	    fromNode = ((ProtoUseNode)fromNode).orgNode;
	}
	// TODO: test field name and type
	if (toNode instanceof ProtoUseNode) {
	    toNode = ((ProtoUseNode)toNode).orgNode;
	}
	// TODO: test field name and type
	routes.addElement(new Route(fromNode, fromEventOut, toNode, toEventIn));
    }

    synchronized boolean setupIsMap(Field nodeField, String protoFieldName) {
	protoFieldName = Field.baseName(protoFieldName);
	Field 	protoField = (Field)FieldSpec.get(protoFieldName);
	if (protoField == null) {
	    System.err.println("Proto.setupIsMap(): Can't find proto field \""+ 
		protoFieldName + "\"");
	    return false;
	} 
	nodeField.update(protoField); // set the initial value

	// Add routes for each way the proto field implies (timestamps will 
	// avoid loops) (( really? ))
	if (protoField.isEventIn()) {
	    if (nodeField.isEventIn()) {
		routes.addElement(
		    new Route(null, protoFieldName, 
			      nodeField.ownerNode, nodeField.fieldName));
	    } else {
		return false;
	    }
	} 
	if (protoField.isEventOut()) {
	    if (nodeField.isEventOut()) {
		routes.addElement(
		    new Route(nodeField.ownerNode, nodeField.fieldName,
			      null, protoFieldName));
	    } else {
		return false;
	    }
	}
	// If the proto field is a plain field, build the EventIn route
	// to map the initial value from the parse.  There won't
	// be any further updates since you can't route to the field 
	if (!protoField.isEventIn() && !protoField.isEventOut()) {
	    routes.addElement(new Route(null, protoFieldName, 
			        nodeField.ownerNode, nodeField.fieldName));
	}
	return true;
    }

    public boolean setupEventIsMap(Node node, String nodeEventName, 
		String protoFieldName) {
	protoFieldName = Field.baseName(protoFieldName);
	Field 	protoField = (Field)FieldSpec.get(protoFieldName);
	if (protoField == null) {
	    System.err.println("Proto.setupEventIsMap: Can't find protoField " +
		protoFieldName);
	    return false;
	} 
	nodeEventName =	Field.baseName(nodeEventName);
	Field 	nodeField = (Field)node.FieldSpec.get(nodeEventName);
	if (nodeField == null) {
	    System.err.println("Proto.setupEventIsMap: Can't find " +
	    	"nodeEventName " +
	    	nodeEventName);
	    return false;
	} 
	// Add routes for each way the proto field implies (timestamps will 
	// avoid loops) (( but cause proto bugs if too fast ))
	if (protoField.isEventIn()) {
	    if (nodeField.isEventIn()) {
		routes.addElement(
		    new Route(null, protoFieldName, 
			      nodeField.ownerNode, nodeEventName));
	    } else {
		return false;
	    }
	} 
	if (protoField.isEventOut()) {
	    if (nodeField.isEventOut()) {
		routes.addElement(
		    new Route(nodeField.ownerNode, nodeEventName,
			      null, protoFieldName));
	    } else {
		return false;
	    }
	}
	return true;
    }

    public void addObject(BaseNode node) {
	nodes.addElement(node);
    }


    // generate a copy of the proto, resolving linkages to the new instance
    public ProtoInstance instance() {

	// create a new Proto and register it with the loader 
	newInstance = new ProtoInstance(loader, name, this);
	if (loader.debug)
	    System.out.println("Proto.instance() begins newInstance is " +
		newInstance.toStringId());
	loader.beginProtoInstance(newInstance);

	// set up the new fields 
	newInstance.initFields();

	// clone the trees attached to the Proto.  Def'd nodes will 
	// register with the the new instance (see also SFNode.clone() and
	// MFNode.clone()).  Use'd nodes will lookup in the new instance (see
	// ProtoUseNode.clone()
	BaseNode retVal = null;
	for (int i = 0; i < nodes.size(); i++) {
	    BaseNode baseNode = (BaseNode) nodes.elementAt(i);
	    BaseNode clone = (BaseNode)baseNode.clone();
	    loader.cleanUp();
	    if (i == 0) {
		newInstance.instanceNode = clone;
	    }
	    loader.registerClone(baseNode, clone);
	}
	loader.cleanUp();
	newInstance.applyRoutes();
	if (loader.debug) 
	    System.out.println("Proto.instance() returns instance with " +
		"VRML tree:\n" + 
		newInstance.instanceNode);
        loader.endProtoInstance();
	return newInstance;
    }

    void registerClone(BaseNode org, BaseNode clone) {
	//System.out.println("registerClone org = " + org.toStringId() + 
        //		" clone = " + clone.toStringId());
	newInstance.nodeMapping.put(org, clone);
    }

    public void setEventIn(String eventInName, Field f) {
	String fieldName = Field.baseName(eventInName);
	f.init(null, FieldSpec, Field.EVENT_IN, fieldName);
    }	

    public void setEventOut(String eventOutName, Field f) {
	String fieldName = Field.baseName(eventOutName);
	f.init(null, FieldSpec, Field.EVENT_OUT, fieldName);
    }
    
    public void setField(String fieldName, Field f) {
	if (loader.debug) {
	    System.out.println("Proto.setField(): adding field named " + 
	    fieldName);
	}
	fieldName = Field.baseName(fieldName);
	f.init(null, FieldSpec, Field.FIELD, fieldName);
    }

    public void setExposedField(String fieldName, Field f) {
	fieldName = Field.baseName(fieldName);
	if (loader.debug) {
	    System.out.println(
		"Proto.setExposedField(): adding field named \"" + fieldName + 
		"\"");
	}
	f.init(null, FieldSpec, Field.EXPOSED_FIELD, fieldName);
    }

    public String toString() {
	String retval = "Proto " + name + " [\n";
	for (Enumeration e = FieldSpec.keys(); e.hasMoreElements();) {
	    String fieldName = (String)e.nextElement();
	    Field  field = (Field)FieldSpec.get(fieldName);
	    retval += "field "; 
	    String fieldType = field.getClass().getName() + " ";
	    int start = fieldType.lastIndexOf(".") + 1;
	    int end = fieldType.length();
	    fieldType = fieldType.substring(start, end);
	    retval += fieldType + " " + fieldName + " " + field;
	}
	retval += "]\n{\n";
	for (int i = 0; i < nodes.size(); i++) {
	    BaseNode baseNode = (BaseNode) nodes.elementAt(i);
	    retval += baseNode + "\n";
	}
	retval += "}\n";
	return retval;
    }
}
