/*
 *      @(#)SFVec3f.java 1.19 99/02/09 17:26:42
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.sun.j3d.loaders.vrml97.impl;
import java.util.Observer;
import java.util.Observable;

public class SFVec3f extends Field {

    float[] value = new float[3];
    float[] initValue = new float[3];

    private void setInit() {
	initValue[0] = value[0];
	initValue[1] = value[1];
	initValue[2] = value[2];
    }

    public SFVec3f(float[] setVal) {
	setValue(setVal);
	setInit();
    }

    public SFVec3f(float x, float y, float z) {
	value[0]=x;
	value[1]=y;
	value[2]=z;
	setInit();
    }

    public SFVec3f() {
	value[0]= 0.0f;
	value[1]= 0.0f;
	value[2]= 0.0f;
	setInit();
    }

    public void reset() {
	value[0] = initValue[0];
	value[1] = initValue[1];
	value[2] = initValue[2];
    }

    public void getValue(float[] vec) {
	vec[0] = value[0];
	vec[1] = value[1];
	vec[2] = value[2];
    }

    public float getX() {
	return value[0];
    }

    public float getY() {
	return value[1];
    }

    public float getZ() {
	return value[2];
    }

    public float[] getValue() { 
       // the ConstSFVec3f does the clone. not here.
	return value;
    }

    public void setValue(float[] v) {
	value[0] = v[0];
	value[1] = v[1];
	value[2] = v[2];
	route();
    }

    public void setValue(float x, float y, float z) {
	value[0] = x;
	value[1] = y;
	value[2] = z;	

	route();
    }

    public void setValue(ConstSFVec3f v) {
	setValue( ((SFVec3f)v.ownerField).value );
    }

    public void setValue(SFVec3f v) {
	setValue( v.value );
    }

    public synchronized Object clone() {
	return new SFVec3f( value );
    }

    synchronized void update(Field field) {
	setValue(((SFVec3f)field).value );
    }

    synchronized ConstField constify() {
	if (constField == null) {
	    constField = new ConstSFVec3f(this);
	}
	return constField;
    }

    public vrml.Field wrap() {
        return new vrml.field.SFVec3f(this);
    }

    public String toString() {
	return value[0] + " " + value[1] + " " + value[2] + "\n";
    }
}
