/*
 *      @(#)Scene.java 1.14 98/11/05 20:35:46
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */


package com.sun.j3d.loaders.vrml97.impl;

import java.util.Hashtable;
import java.util.Vector;
import javax.media.j3d.SharedGroup;

/** This is an internal class used by the Loader.
 * <p>
 * Collects elements loaded by the Loader into a lists 
 */
public class Scene implements Namespace {
    public Vector	objects = new Vector();;
    public Vector	viewpoints = new Vector();
    public Vector	navInfos = new Vector();
    public WorldInfo    worldInfo;
    public Vector	backgrounds = new Vector();
    public Vector	fogs = new Vector();
    public Vector	lights = new Vector();
    public Vector	sharedGroups = new Vector();
    public Vector	timeSensors = new Vector();
    public Vector	visibilitySensors = new Vector();
    public Vector	touchSensors = new Vector();
    public Vector	audioClips = new Vector();
    public Hashtable   	defTable = new Hashtable();
    public Proto	firstProto = null;
    public Hashtable 	protos = new Hashtable();
    public String	description = null;
    public int		numTris;

    void addObject(com.sun.j3d.loaders.vrml97.impl.BaseNode object) {
	if (object != null) {
	    objects.addElement(object);   
	    numTris += object.getNumTris(); 
	}
    };
    void addViewpoint(Viewpoint viewpoint) {
        viewpoints.addElement(viewpoint);   
    };
    void addNavigationInfo(NavigationInfo navInfo) {
        navInfos.addElement(navInfo);   
    };
    void addBackground(Background background) {
        backgrounds.addElement(background);   
    };
    void addFog(Fog fog) {
        fogs.addElement(fog);   
    };
    void addLight(Light light) {
        lights.addElement(light);   
    };
    void addSharedGroup(SharedGroup sg) {
        sharedGroups.addElement(sg);   
    };
    void addTimeSensor(TimeSensor ts) {
        timeSensors.addElement(ts);   
    };
    void addVisibilitySensor(VisibilitySensor vs) {
        visibilitySensors.addElement(vs);   
    };
    void addTouchSensor(TouchSensor ts) {
        touchSensors.addElement(ts);   
    };
    void addAudioClip(AudioClip ac) {
        audioClips.addElement(ac);   
    };

    // for now, worldInfo is a single field, ie not a
    // bindable node or stack/vector.
    void setWorldInfo(WorldInfo wi) {
	// cant set the description, since at this point,
	// the parser hasn't read the fields yet. we only
	// know were done parsing within the browser.doParse();
	worldInfo = wi;
    }

    void setDescription(String desc) {
	description = desc;
    };
    // these are the def/use for the scene's namespace
    public void define(String defName, BaseNode node) {
        defTable.put(defName, node);
    }
    public BaseNode use(String defName) {
        return (BaseNode) defTable.get(defName);
    }
}
