/*
 *      @(#)Script.java 1.24 99/02/18 13:54:36
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.sun.j3d.loaders.vrml97.impl;

import java.util.Hashtable;
import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.MalformedURLException;
import java.util.StringTokenizer;

public class Script extends BaseNode implements Notifier
{
	public Hashtable FieldSpec = new Hashtable(24);
	vrml.node.Script specScript;
	com.sun.j3d.loaders.vrml97.impl.Event ievent;
	vrml.Event event;

	// Exposed Field
	MFString url; // currently unused 
	
	// field
	SFBool directOutput;
	SFBool mustEvaluate;
	
	// the byteCodes are resolved by the Parser's class loader to 
	// an instance of vrml.node.Script 

	public Script(Loader loader) {
	    super(loader);
	    ievent = new com.sun.j3d.loaders.vrml97.impl.Event("default",
		    0.0,(ConstField)null);
	    event = new vrml.Event(ievent);
	    
	    url = new MFString();	
	    directOutput = new SFBool(false);	
	    mustEvaluate = new SFBool(false);
	    initFields();
	}

	void setByteCodes(vrml.node.Script byteCodes) {
		specScript = byteCodes;
	}
		
	public final Field getEventIn(String eventInName) 
		throws vrml.InvalidEventInException {

	    Field f = (Field)FieldSpec.get(eventInName);
	    if (( f == null ) || !f.isEventIn()) {
		throw new vrml.InvalidEventInException();
	    }
	    return f;
	}

        public Field getEventOut(String eventOutName) 
	    throws vrml.InvalidEventOutException {

	    Field f = (Field)FieldSpec.get(eventOutName);
	    if ((f == null) || !f.isEventOut()) {
		throw new vrml.InvalidEventOutException();
	    }
	    return f;
        }
        

        public final Field getField(String fieldName){           
	    fieldName = Field.baseName(fieldName);
	    Field f = (Field)FieldSpec.get(fieldName);         
	    if ( f == null ) {
		throw new vrml.InvalidFieldException();
	    }
	    return f;
        }               

	void initImpl() {
	    URL u; 
	    URLClassLoader scl; 
	    String scriptName = "";
	    String urlName;
	    StringTokenizer tok; 
	    int countToks; 

	    scriptName = url.strings[0];
	    // strip off ".class"
	    if (scriptName.endsWith(".class")) scriptName = scriptName.substring(0,scriptName.length()-6);

	    if (loader.debug) {
		System.out.println("Script.initImpl() called");
	    }
	    if (scriptName.startsWith("vrmlscript:") || 
			scriptName.startsWith("javascript:") ||
			scriptName.startsWith("ecma:") ||
			scriptName.startsWith("ECMA:") ||
			scriptName.startsWith("excema")) {
		 throw new vrml.InvalidVRMLSyntaxException("Script nodes are faster in Java bytecode format and more powerful");
	    } else {
		scl = loader.scl;
		try {
		    setByteCodes((vrml.node.Script)scl.loadClass(scriptName).newInstance());
		} catch (java.lang.ClassNotFoundException cnfe) {
		    throw new vrml.InvalidVRMLSyntaxException("Unable to load class "+scriptName);
		} catch (java.lang.InstantiationException ie) {
		    throw new vrml.InvalidVRMLSyntaxException("Unable to intance script node from data");
		} catch (java.lang.IllegalAccessException iae) {
		    throw new vrml.InvalidVRMLSyntaxException("Illegal access to script node data");
		} 



		// these used to be in initialize()
		specScript.registerOwner(this);
		specScript.initialize();
	    }
	    implReady = true;
	}


	public void notifyMethod(String eventInName, double time) { 	
	    if (loader.debug) {
		System.out.println("Script.notifyMethod(" + eventInName + ")");
	    }
	    if (eventInName.equals("url")) {
		// TODO: implement 
	    } else if (eventInName.startsWith("route_")) {
		// Noop
	    } else {
		if (loader.debug) {
		   System.out.println("Passing event to script");
		}
		ievent.name = eventInName;
		// the user sees an event with a const field in value
		ievent.value = (getField(eventInName)).constify();
		ievent.timeStamp = time;
		specScript.processEvent( event );
		specScript.eventsProcessed();
	    }
	}
	
	// there is a processEvents(int count, Event[] events)
	// method but we don't think at this time that scripts
	// will use it.
		
	public String getType() {
	    return "Script";
	}

	public Object clone() {
	    Script s;
	    String name;
	    s = new Script(loader);
	    s.initFields();
	    name = (String)((Object)specScript.toString());
	    try {
		s.setByteCodes(loader.newInstance(
			name.substring(0,name.indexOf('@'))));
	    } catch (IllegalAccessException iae) {
		    iae.printStackTrace();
	    } catch (InstantiationException ie) {
		    ie.printStackTrace();
	    } catch (ClassNotFoundException cnfe) {
		    cnfe.printStackTrace();
	    }
	    return (Object)s;
	}

	public synchronized void setEventIn(String eventInName, Field f) {
	    if (loader.debug) {
		System.out.println("Adding eventIn " + f.toStringId() + 
			"\n    with name " + eventInName + 
			"\n    to script node " + this.toStringId());
	    }
	    f.init(this, FieldSpec, Field.EVENT_IN, eventInName);
	}	

	public synchronized void setEventOut(String eventOutName, Field f) {
	    if (loader.debug) {
		System.out.println("Adding eventOut " + f.toStringId() + 
			"\n    with name " + eventOutName + 
			"\n    to script node " + this.toStringId());
	    }
	    f.init(this, FieldSpec, Field.EVENT_OUT, eventOutName);
	}
	
	public synchronized void setField(String fieldName, Field f) {
	    if (loader.debug) {
		System.out.println("Adding field " + f.toStringId() + 
			"\n    with name " + fieldName + 
			"\n    to script node " + this.toStringId());
	    }
	    f.init(this, FieldSpec, Field.FIELD, fieldName);
	}

	public com.sun.j3d.loaders.vrml97.impl.Browser getBrowser() { 
	    return browser; 
	}

	void initFields() {
	    url.init(this, FieldSpec, Field.EXPOSED_FIELD, "url");
	    directOutput.init(this, FieldSpec, Field.FIELD, "directOutput");
	    mustEvaluate.init(this, FieldSpec, Field.FIELD, "mustEvaluate");
	}

        public vrml.BaseNode wrap() {
	    return new vrml.node.Script(this);
        }

}
