/*
 *      @(#)Text.java 1.16 98/11/05 20:35:24
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.sun.j3d.loaders.vrml97.impl;

import com.sun.j3d.utils.geometry.Text2D;
import java.awt.Font;
import javax.vecmath.Color3f;
import javax.media.j3d.Shape3D;

import javax.media.j3d.Billboard;
import javax.media.j3d.TransformGroup;

public class Text extends Geometry {

    // exposedField
    MFString string;
    SFNode fontStyle;
    MFFloat length;
    SFFloat maxExtent;

    Text2D impl = null;
    Billboard billboard;

    public Text(Loader loader) {
	super(loader);
	string = new MFString(); 
	fontStyle = new SFNode();
	fontStyle.node = new FontStyle(loader);
	length = new MFFloat();
	maxExtent = new SFFloat(0.0f);
	initFields();
    }

    public Text(Loader loader, MFString string, 
	SFNode fontStyle, MFFloat length,
	SFFloat maxExtent ) {

	super(loader);
	this.string = string;
	this.fontStyle = fontStyle;
	this.length = length;
	this.maxExtent = maxExtent;
	initFields();
    }

    Shape3D createText2D() {
	if (loader.debug) {
	    System.out.println("Text.createText2D() called");
	}
        if ((string.strings != null) && (string.strings.length > 0)) {
	    if (loader.debug) {
		System.out.println("creating Text2D object");
	    }
	    impl = new Text2D(string.strings[0], new Color3f(1.0f, 1.0f, 1.0f), 
		    "Serif", 60, Font.BOLD);
	    implNode = (javax.media.j3d.Node) impl;
	}
	return impl;
    }

    public javax.media.j3d.Geometry getImplGeom() {
	return null;
    }
    public boolean haveTexture() {  return false; };
    public int getNumTris() { return 0; };
    public javax.media.j3d.BoundingBox getBoundingBox() { return null; };


    public String getType() { return "Text"; }

    public Object clone() {
	return new Text(loader, (MFString)string.clone(), 
	    (SFNode)fontStyle.clone(), (MFFloat)length.clone(), 
	    (SFFloat)maxExtent.clone() );
    }

    public void notifyMethod(String eventInName, double time) {
	System.out.println("Text: unimplemented event " + eventInName);
    }

    void initFields() {
	string.init(this, FieldSpec, Field.EXPOSED_FIELD, "string");
	fontStyle.init(this, FieldSpec, Field.EXPOSED_FIELD, "fontStyle");
	length.init(this, FieldSpec, Field.EXPOSED_FIELD, "length");
	maxExtent.init(this, FieldSpec, Field.EXPOSED_FIELD, "maxExtent");
    }
}
