/******************************************************************************
 *
 *                      VRML Browser basic classes
 *                   For External Authoring Interface
 *
 *                   (C) The VRML Consortium
 *
 *  Written by Justin Couch: justin@vlc.com.au
 *
 *****************************************************************************/

package vrml.external;

import java.applet.Applet;
import vrml.external.exception.*;

/**
 * Basic browser interface that represents the interface to the VRML browser
 * from any application. Individual VRML browser implementors are to extend this
 * interface and provide this functionality. The individual users will not see
 * anything but this interface.
 * <P>
 * A number of the methods in this applicationcan take strings representing URLs.
 * Relative URL strings contained in URL fields of nodes or these method
 * arguments are interpreted relative to the base document of the loadURL call.
  * <P>
 *
 * @deprecated
 * @version 1.0
 */
public class Browser
{
  /**
   * Get a browser from the given java applet reference as a base in the
   * current HTML page. Used when attempting to access a browser on the current
   * page as this applet and is the first browser on the page. Generically, the
   * same as calling getBrowser(applet, "", 0);
   *
   * @param applet The applet reference to use
   * @return A reference to the Browser implementation
   */
  public static Browser getBrowser(Applet applet)
  {
    return null;
  }

  /**
   * Get a browser from the given java applet reference one some named page and
   * at some embbed location. Used when attempting to access a browser on
   * another HTML page within a multi-framed environment, or if there are a
   * number of VRML browser instances located on the same page.
   * <P>
   * If the frame name is a zero length string or null then it is assumed to be
   * located on the same HTML page as the applet. The index is the number of
   * the embbed VRML browser starting from the top of the page. If there are
   * other non-VRML plugins embedded in the page these are not taken into
   * account in calculating the embed index.
   *
   * @param applet The applet reference to use
   * @param frameName The name of the frame to look into for the browser
   * @param index The embed index of the VRML browser in the page
   * @return A reference to the Browser implementation
   */
  public static Browser getBrowser(Applet applet, String frameName, int index)
  {
    return null;
  }

  /**
   * Get the name of the browser. The name is an implementation specific
   * string representing the browser.
   *
   * @return The name of the browser or null if not supported
   */
  public String getName()
  {
    return "Default Deprecated Browser";
  }

  /**
   * Get the version of the browser. Returns an implementation specific
   * representation of the version number.
   *
   * @return The version of the browser or null if not supported
   */
  public String getVersion()
  {
    return null;
  }

  /**
   * Get the current velocity of the bound viewpoint in meters per second.
   * The velocity is defined in terms of the world values, not the local
   * coordinate system of the viewpoint.
   *
   * @return The velocity in m/s or 0.0 if not supported
   */
  public float getCurrentSpeed()
  {
    return 0;
  }

  /**
   * Get the current frame rate of the browser in frames per second.
   *
   * @return The current frame rate or 0.0 if not supported
   */
  public float getCurrentFrameRate()
  {
    return 0;
  }

  /**
   * Get the fully qualified URL of the currently loaded world. This returns
   * the entire URL including any possible arguments that might be associated
   * with a CGI call or similar mechanism. If the initial world is replaced
   * with <CODE>loadURL</CODE> then the string will reflect the new URL. If
   * <CODE>replaceWorld</CODE> is called then the URL still represents the
   * original world.
   *
   * @return A string of the URL or null if not supported.
   * @see #loadURL
   * @see #replaceWorld
   */
  public String getWorldURL()
  {
    return null;
  }

  /**
   * Replace the current world with the given nodes. Replaces the entire
   * contents of the VRML world with the new nodes. Any node references that
   * belonged to the previous world are still valid but no longer form part of
   * the scene graph (unless it is these nodes passed to this method). The
   * URL of the world still represents the just unloaded world.
   * <P>
   * Calling this method causes a SHUTDOWN event followed by an INITIALIZED
   * event to be generated.
   *
   * @param nodes The list of nodes to use as the new root of the world
   * @exception IllegalArgumentException if the nodes are not valid VRML nodes
   */
  public void replaceWorld(Node[] nodes)
    throws IllegalArgumentException
  {
  }

  /**
   * Load the URL as the new root of the scene. Replaces all the current
   * scene graph with the new world. A non-blocking call that will change the
   * contents at some time in the future.
   * <P>
   * Generates an immediate SHUTDOWN event and then when the new contents are
   * ready to be loaded, sends an INITIALIZED event.
   *
   * @param url The list of URLs in decreasing order of preference as defined
   *   in the VRML97 specification.
   * @param paramaters The list of parameters to accompany the load call as
   *   defined in the Anchor node specification of VRML97
   */
  public void loadURL(String[] url, String[] parameter)
  {
  }

  /**
   * Set the description of the current world. If the world is operating as
   * part of a web browser then it shall attempt to set the title of the
   * window. If the browser is from a component then the result is dependent
   * on the implementation
   *
   * @param desc The description string to set.
   */
  public void setDescription(String desc)
  {
  }

  /**
   * Parse the given string and turn this into a list of VRML nodes. Method
   * is a blocking call that won't return until all of the top level nodes
   * defined in the string have been returned.
   * <P>
   * At the point that this method returns, external files such as textures,
   * sounds and inlines may not have been loaded.
   * <P>
   * The string may contain all legal VRML syntax. The VRML header line is not
   * required to be present in the string.
   *
   * @param vrmlString The string containing VRML string syntax
   * @return A list of the top level nodes in VRML representation as defined
   *    in the parameter
   * @exception InvalidVrmlException If the string does not contain legal
   *   VRML syntax or no node instantiations
   */
  public Node[] createVrmlFromString(String vrmlString)
    throws InvalidVrmlException
  {
    return null;
  }

  /**
   * Create and load VRML from the given URL and place the returned values
   * as nodes into the given VRML node in the scene. The difference between
   * this and loadURL is that this method does not replace the entire scene
   * with the contents from the URL. Instead, it places the return values
   * as events in the nominated node and MFNode eventIn.
   *
   * @param url The list of URLs in decreasing order of preference as defined
   *   in the VRML97 specification.
   * @param node The destination node for the VRML code to be sent to.
   * @param eventIn The name of the MFNode eventIn to send the nodes to.
   */
  public void createVrmlFromURL(String[] url, Node node, String eventIn)
  {
  }

  /**
   * Get a DEF node by name. Nodes given DEF names in the root scene graph
   * are available to be retrieved by this method. DEFed nodes in Inlines,
   * createVrmlFromString and createVrmlFromURL are not available.
   *
   * @param name The name of the DEF node to retrieve
   * @return A reference to that node
   * @exception InvalidNodeException The named node does not exist or is not
   *    accessible.
   */
  public Node getNode(String name)
      throws InvalidNodeException
  {
    return null;
  }

  /**
   * Add a route between two nodes, from an eventOut to an eventIn. If the
   * ROUTE already exists, this method silently exits. It does not attempt
   * to add a second parallel ROUTE.
   *
   * @param fromNode The source node for the route
   * @param eventOut The eventOut source of the route
   * @param to Node The destination node of the route
   * @param eventIn The eventIn destination of the route
   * @exception IllegalArgumentException if the named fields or node do not
   *     exist
   */
  public void addRoute(Node fromNode, String eventOut,
                       Node toNode,   String eventIn)
      throws IllegalArgumentException
  {
  }

  /**
   * Delete a route between two nodes. If the route does not exist, the
   * method silently exits.
   *
   * @param fromNode The source node for the route
   * @param eventOut The eventOut source of the route
   * @param to Node The destination node of the route
   * @param eventIn The eventIn destination of the route
   * @exception IllegalArgumentException if the named fields or node do not
   *     exist
   */
  public void deleteRoute(Node fromNode, String eventOut,
                          Node toNode,   String eventIn)
      throws IllegalArgumentException
  {
  }
}